@@ -22,7 +22,7 @@ impl HtmlPresser {
         origin: &str,
         link_limit: usize,
         page_limit: usize,
-    ) -> Result<Vec<String>, std::io::Error> {
+    ) -> Result<Vec<String>, Error> {
         let mut urls: Vec<String> = vec![origin.to_string()];
         let mut indexer: usize = 0;
 
@@ -35,16 +35,18 @@ impl HtmlPresser {
                 }
             };
 
-            if let Some(domain_anchors) = record.domain_anchors() {
-                for anchor in domain_anchors {
-                    if urls.contains(&anchor) {
-                        continue;
-                    } else {
-                        urls.push(anchor)
-                    }
+            let domain_anchors = record.domain_anchors();
+
+            indexer += 1;
+
+            for anchor in domain_anchors {
+                if urls.contains(&anchor) {
+                    continue;
+                } else {
+                    urls.push(anchor)
                 }
             }
-            indexer += 1;
+
             if urls.len() > link_limit || indexer >= page_limit || indexer >= urls.len() {
                 break;
             }
@@ -71,9 +73,11 @@ impl HtmlPresser {
         pattern: &str,
         link_limit: usize,
         page_limit: usize,
-    ) -> Result<Vec<String>, std::io::Error> {
+    ) -> Result<Vec<String>, Error> {
         let mut urls: Vec<String> = vec![origin.to_string()];
 
+        let mut curated_links: Vec<String> = vec![];
+
         let mut indexer: usize = 0;
 
         loop {
@@ -85,22 +89,24 @@ impl HtmlPresser {
                 }
             };
 
-            if let Some(curated_anchors) = document.anchors_curate(pattern) {
-                for anchor in curated_anchors {
-                    if urls.contains(&anchor) {
-                        continue;
-                    } else {
-                        urls.push(anchor);
-                    }
+            let curated_anchors = document.anchors_curate(pattern);
+            indexer += 1;
+
+            for anchor in curated_anchors {
+                if urls.contains(&anchor) {
+                    continue;
+                } else {
+                    urls.push(anchor.clone());
+                    curated_links.push(anchor);
                 }
             }
-            indexer += 1;
+
             if urls.len() > link_limit || indexer >= page_limit || indexer >= urls.len() {
                 break;
             }
         }
 
-        Ok(urls)
+        Ok(curated_links)
     }
 
     /// public function
@@ -108,10 +114,7 @@ impl HtmlPresser {
     /// returns a result of Ok if good and std::io::Error
     /// meant to just crawl a site and get html records to a blind depth
     /// uses HTTP
-    pub async fn press_records_blind(
-        origin: &str,
-        limit: usize,
-    ) -> Result<Vec<HtmlRecord>, std::io::Error> {
+    pub async fn press_records_blind(origin: &str, limit: usize) -> Result<Vec<HtmlRecord>, Error> {
         let mut html_records: Vec<HtmlRecord> = vec![];
         let mut urls: Vec<String> = vec![origin.to_string()];
         let mut indexer: usize = 0;
@@ -129,17 +132,17 @@ impl HtmlPresser {
                 }
             };
 
-            if let Some(domain_anchors) = record.domain_anchors() {
-                for anchor in domain_anchors {
-                    if urls.contains(&anchor) {
-                        continue;
-                    } else {
-                        urls.push(anchor)
-                    }
-                }
-            }
+            let domain_anchors = record.domain_anchors();
+
             indexer += 1;
 
+            for anchor in domain_anchors {
+                if urls.contains(&anchor) {
+                    continue;
+                } else {
+                    urls.push(anchor)
+                }
+            }
             html_records.push(record);
             if indexer >= limit || indexer >= urls.len() {
                 break;
@@ -153,7 +156,7 @@ impl HtmlPresser {
     ///takes in a vector of url strings that need to be harvested
     ///outputs result Ok(Vec<HtmlRecord>)
     /// uses HTTP
-    pub async fn press_records(links: Vec<&str>) -> Result<Vec<HtmlRecord>, std::io::Error> {
+    pub async fn press_records(links: Vec<&str>) -> Result<Vec<HtmlRecord>, Error> {
         let mut html_records: Vec<HtmlRecord> = vec![];
 
         for link in links {
@@ -168,7 +171,7 @@ impl HtmlPresser {
     /// takes in a string reference to a url to be pressed
     ///outputs an empty ok and result
     /// uses HTTP
-    pub async fn press_record(link: &str) -> Result<HtmlRecord, std::io::Error> {
+    pub async fn press_record(link: &str) -> Result<HtmlRecord, Error> {
         let record = fetch_html_record(link).await.unwrap();
         Ok(record)
     }
@@ -196,17 +199,17 @@ impl HtmlPresser {
                 }
             };
 
-            if let Some(curated_anchors) = record.anchors_curate(pattern) {
-                record_vec.push(record);
+            let curated_anchors = record.anchors_curate(pattern);
 
-                indexer += 1;
+            record_vec.push(record);
 
-                for anchor in curated_anchors {
-                    if links.contains(&anchor) {
-                        continue;
-                    } else {
-                        links.push(anchor)
-                    }
+            indexer += 1;
+
+            for anchor in curated_anchors {
+                if links.contains(&anchor) {
+                    continue;
+                } else {
+                    links.push(anchor)
                 }
             }
 
