@@ -1,5 +1,4 @@
 use crate::constants::EXTENTIONS;
-use lazy_static::lazy_static;
 use regex::Regex;
 use scraper::{Html, Selector};
 use std::collections::{HashMap, HashSet};
@@ -12,7 +11,6 @@ pub struct HtmlRecord {
     pub origin: String,
     pub headers: HashMap<String, Vec<String>>,
     pub body: String,
-    html: Html,
 }
 
 /*
@@ -22,29 +20,20 @@ HTML DOCUMENT
 */
 
 impl HtmlRecord {
-    pub fn new(origin: String, headers: HashMap<String, Vec<String>>, body: String) -> HtmlRecord {
-        HtmlRecord {
-            origin,
-            headers,
-            body: body.clone(),
-            html: Html::parse_document(&body),
-        }
-    }
-
     /// public function,
     /// input is a reference to self.
     /// returns a HashSet<String>,
     /// transforms body of HtmlRecord to collect all anchors.
-    pub fn anchors(&self) -> Option<HashSet<String>> {
+    pub fn anchors(&self) -> HashSet<String> {
         let mut ret_vec: Vec<String> = vec![];
+
+        let doc = Html::parse_document(self.body.as_str());
         let selector = Selector::parse("a").unwrap();
-        //let html = self.get_html(); //Html::parse_document(&self.body);
-        for element in self.html.select(&selector) {
+        for element in doc.select(&selector) {
             match element.value().attr("href") {
                 Some(link) => {
-                    if let Ok(parse_link) = HtmlRecord::check_link(&self.origin, link) {
-                        ret_vec.push(parse_link)
-                    }
+                    let link = HtmlRecord::check_link(&self.origin, link);
+                    ret_vec.push(link)
                 }
                 None => continue,
             };
@@ -52,31 +41,27 @@ impl HtmlRecord {
 
         let link_hashset: HashSet<String> = ret_vec.iter().cloned().collect();
 
-        if link_hashset.is_empty() {
-            None
-        } else {
-            Some(link_hashset)
-        }
+        link_hashset
     }
 
     /// public function:
     /// input is a reference to self.
-    /// returns an Option<HashSet<String>>,
+    /// returns a HashSet<String>,
     /// transforms body of HtmlRecord to collect domain related anchors.
-    pub fn domain_anchors(&self) -> Option<HashSet<String>> {
+    pub fn domain_anchors(&self) -> HashSet<String> {
         let mut ret_vec: Vec<String> = vec![];
+        let doc = Html::parse_document(self.body.as_str());
         let selector = Selector::parse("a").unwrap();
-        //let html = self.get_html();
-        for element in self.html.select(&selector) {
+
+        for element in doc.select(&selector) {
             match element.value().attr("href") {
                 Some(link) => {
-                    if let Ok(parsed_link) = HtmlRecord::check_link(&self.origin, link) {
-                        if HtmlRecord::is_domain_related(&self.origin, &parsed_link)
-                            && HtmlRecord::is_http(&parsed_link)
-                            && !HtmlRecord::has_extension(&parsed_link)
-                        {
-                            ret_vec.push(parsed_link)
-                        }
+                    let link = HtmlRecord::check_link(&self.origin, link);
+                    if HtmlRecord::is_domain_related(&self.origin, &link)
+                        && HtmlRecord::is_http(&link)
+                        && !HtmlRecord::has_extension(&link)
+                    {
+                        ret_vec.push(link)
                     }
                 }
                 None => continue,
@@ -85,31 +70,27 @@ impl HtmlRecord {
 
         let link_hashset: HashSet<String> = ret_vec.iter().cloned().collect();
 
-        if link_hashset.is_empty() {
-            None
-        } else {
-            Some(link_hashset)
-        }
+        link_hashset
     }
 
     /// public function:
     /// input is a reference to self.
     /// returns a HashSet<String>
     /// transforms body of HtmlRecord to collect non domain related anchors
-    pub fn non_domain_anchors(&self) -> Option<HashSet<String>> {
+    pub fn non_domain_anchors(&self) -> HashSet<String> {
         let mut ret_vec: Vec<String> = vec![];
+        let doc = Html::parse_document(self.body.as_str());
         let selector = Selector::parse("a").unwrap();
 
-        for element in self.html.select(&selector) {
+        for element in doc.select(&selector) {
             match element.value().attr("href") {
                 Some(link) => {
-                    if let Ok(link) = HtmlRecord::check_link(&self.origin, link) {
-                        if !HtmlRecord::is_domain_related(&self.origin, &link)
-                            && HtmlRecord::is_http(&link)
-                            && !HtmlRecord::has_extension(&link)
-                        {
-                            ret_vec.push(link)
-                        }
+                    let link = HtmlRecord::check_link(&self.origin, link);
+                    if !HtmlRecord::is_domain_related(&self.origin, &link)
+                        && HtmlRecord::is_http(&link)
+                        && !HtmlRecord::has_extension(&link)
+                    {
+                        ret_vec.push(link)
                     }
                 }
                 None => continue,
@@ -118,126 +99,90 @@ impl HtmlRecord {
 
         let link_hashset: HashSet<String> = ret_vec.iter().cloned().collect();
 
-        if link_hashset.is_empty() {
-            None
-        } else {
-            Some(link_hashset)
-        }
+        link_hashset
     }
 
     ///public function:
     /// inputs: &self and regex &str.
     /// returns a hashset<string>,
     /// used for when trying to crawl things like online threads.
-    pub fn anchors_curate(&self, regex: &str) -> Option<HashSet<String>> {
+    pub fn anchors_curate(&self, regex: &str) -> HashSet<String> {
         let mut ret_vec: Vec<String> = vec![];
-        if let Ok(re) = Regex::new(regex) {
-            let selector = Selector::parse("a").unwrap();
-            for element in self.html.select(&selector) {
-                match element.value().attr("href") {
-                    Some(link) => {
-                        if let Ok(parsed_link) = HtmlRecord::check_link(&self.origin, link) {
-                            if re.is_match(&parsed_link) && !HtmlRecord::has_extension(&parsed_link)
-                            {
-                                ret_vec.push(parsed_link)
-                            }
-                        }
-                    }
-                    None => continue,
-                };
-            }
-            let link_hashset: HashSet<String> = ret_vec.iter().cloned().collect();
+        let re = Regex::new(regex).unwrap();
+        let doc = Html::parse_document(self.body.as_str());
+        let selector = Selector::parse("a").unwrap();
 
-            if link_hashset.is_empty() {
-                None
-            } else {
-                Some(link_hashset)
-            }
-        } else {
-            return None;
+        for element in doc.select(&selector) {
+            match element.value().attr("href") {
+                Some(link) => {
+                    let link = HtmlRecord::check_link(&self.origin, link);
+                    if !HtmlRecord::is_domain_related(&self.origin, &link)
+                        && re.is_match(&link)
+                        && !HtmlRecord::has_extension(&link)
+                    {
+                        ret_vec.push(link)
+                    }
+                }
+                None => continue,
+            };
         }
+        let link_hashset: HashSet<String> = ret_vec.iter().cloned().collect();
+
+        link_hashset
     }
 
     ///public method,
     /// gets the text of a tag.
-    pub fn tag_text(&self, tag: &str) -> Option<Vec<String>> {
+    pub fn tag_text(&self, tag: &str) -> Vec<String> {
         let mut ret_vec: Vec<String> = vec![];
         let re = Regex::new(r"\n|\t").unwrap();
         let selector = Selector::parse(tag).unwrap();
-        for element in self.html.select(&selector) {
+        let doc = Html::parse_document(self.body.as_str());
+
+        for element in doc.select(&selector) {
             let text_raw = element.text().collect::<String>(); //Vec<_>>();
             let text_parsed = re.replace_all(&text_raw, "").to_string();
             if !text_parsed.is_empty() {
                 ret_vec.push(String::from(text_parsed.trim()));
             }
         }
-        if ret_vec.is_empty() {
-            None
-        } else {
-            Some(ret_vec)
-        }
+        ret_vec
     }
 
     ///public method,
     /// gets the html of a tag.
-    pub fn tag_html(&self, tag: &str) -> Option<Vec<String>> {
+    pub fn tag_html(&self, tag: &str) -> Vec<String> {
         let mut ret_vec: Vec<String> = vec![];
         let selector = Selector::parse(tag).unwrap();
-        for element in self.html.select(&selector) {
+        let doc = Html::parse_document(self.body.as_str());
+
+        for element in doc.select(&selector) {
             ret_vec.push(element.html());
         }
-
-        if !ret_vec.is_empty() {
-            Some(ret_vec)
-        } else {
-            None
-        }
+        ret_vec
     }
 
     /// public function:
     /// returns HashMap<String, Vec<String>>.
     /// takes in reference to self,
     /// use to map out the meta information in html string
-    pub fn html_meta(&self) -> Option<HashMap<String, Vec<String>>> {
+    pub fn meta_data(&self) -> HashMap<String, Vec<String>> {
         let mut meta_hash: HashMap<String, Vec<String>> = HashMap::new();
-        let meta_selector = Selector::parse("meta").unwrap();
-        for meta_element in self.html.select(&meta_selector) {
-            let name = meta_element.value().attr("name").unwrap_or("Content-Type");
-            let content = match meta_element.value().attr("content") {
+        let doc = Html::parse_document(self.body.as_str());
+        let selector = Selector::parse("meta").unwrap();
+
+        for element in doc.select(&selector) {
+            let name = element.value().attr("name").unwrap_or("Content-Type");
+            let content = match element.value().attr("content") {
                 Some(content) => Vec::from_iter(content.split(',').map(String::from)),
                 None => vec!["none".to_string()],
             };
+
             meta_hash.insert(String::from(name), content);
         }
+        println!("{:?}", meta_hash);
 
-        if meta_hash.is_empty() {
-            None
-        } else {
-            Some(meta_hash)
-        }
-    }
-
-    ///get_emails()->Option<Vec<String>>
-    ///returns all emails withing a body.
-    pub fn get_emails(&self) -> Option<Vec<String>> {
-        lazy_static! {
-            static ref RE: Regex =
-               Regex::new(r"([a-z0-9_+]([a-z0-9_+.]*[a-z0-9_+])?)@([a-z0-9]+([\-\.]{1}[a-z0-9]+)*\.[a-z]{2,6})").unwrap();
-        }
-        // iterate over all matches
-        let emails: Vec<String> = RE
-            .find_iter(&self.body)
-            // try to parse the string matches as i64 (inferred from fn type signature)
-            // and filter out the matches that can't be parsed (e.g. if there are too many digits to store in an i64).
-            .map(|email| String::from(email.as_str()))
-            // collect the results in to a Vec<i64> (inferred from fn type signature)
-            .collect();
-
-        if emails.is_empty() {
-            None
-        } else {
-            Some(emails)
-        }
+        meta_hash
     }
     /*
 
@@ -256,7 +201,26 @@ impl HtmlRecord {
         let origin_url = Url::parse(check_string).expect("self is not a url");
         let url_to_check = Url::parse(link).expect("link is not a string");
 
-        origin_url.host() == url_to_check.host()
+        let origin_vec = match origin_url.host() {
+            Some(host) => Vec::from_iter(host.to_string().split('.').map(String::from)),
+            None => return false,
+        };
+
+        let checker_vec = match url_to_check.host() {
+            Some(host) => Vec::from_iter(host.to_string().split('.').map(String::from)),
+            None => return false,
+        };
+
+        //google's amazon's
+        let origin_second_level = &origin_vec[origin_vec.len() - 2];
+        let checker_second_level = &checker_vec[checker_vec.len() - 2];
+
+        //com's org's
+        let origin_top_level = &origin_vec[origin_vec.len() - 1];
+        let checker_top_level = &checker_vec[checker_vec.len() - 1];
+
+        //      google         ==         google              com          ==         com
+        origin_second_level == checker_second_level && origin_top_level == checker_top_level
     }
 
     ///private method: check_link
@@ -264,9 +228,9 @@ impl HtmlRecord {
     /// input "in_link", type &str, stands for an HtmlDocument anchor that may need parsing
     /// this is a cursory check to see if a parse is needed
     /// returns string
-    fn check_link(origin: &str, in_link: &str) -> Result<String, std::string::ParseError> {
+    fn check_link(origin: &str, in_link: &str) -> String {
         match Url::parse(in_link) {
-            Ok(link) => Ok(link.to_string()),
+            Ok(link) => link.to_string(),
             Err(_) => HtmlRecord::parse_link(origin, in_link),
         }
     }
@@ -276,9 +240,9 @@ impl HtmlRecord {
     /// the output is the parsed string
     /// use case
     /// get an anchor tag href where it is unparsed, as in "/"
-    fn parse_link(origin: &str, unparsed_link: &str) -> Result<String, std::string::ParseError> {
+    fn parse_link(origin: &str, unparsed_link: &str) -> String {
         let host = Url::parse(origin).expect("origin not a url");
-        let host_string = host.host_str().expect("host is not doable");
+        let host_string = host.host_str().unwrap_or_default().to_string();
 
         let parsed_link = if unparsed_link.substring(0, 1) == "/" {
             format!("{}://{}{}", host.scheme(), host_string, unparsed_link)
@@ -286,7 +250,7 @@ impl HtmlRecord {
             format!("{}://{}/{}", host.scheme(), host_string, unparsed_link)
         };
 
-        Ok(parsed_link)
+        parsed_link
     }
 
     /// private method
